(function () {

    class AdStyle {

        constructor(adUnitId, style, typeName) {
            this.adUnitId = adUnitId;
            this._width = style.width;
            this._height = style.height;
            this._left = style.left;
            this._top = style.top;
            this.typeName = typeName;
        }

        get width() {
            return this._width;
        }

        get height() {
            return this._height;
        }

        set left(left) {
            this._left = left;
            try {
                jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', `set${this.typeName}Left`, '(Ljava/lang/String;I)V', this.adUnitId, left);
            } catch (e) { }
        }

        get left() {
            return this._left
        }

        set top(top) {
            this._top = top;
            try {
                jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', `set${this.typeName}Top`, '(Ljava/lang/String;I)V', this.adUnitId, top);
            } catch (e) { }
        }

        get top() {
            return this._top;
        }
    }

    class BaseAd {

        constructor() {
            this.loadCallbacks = [];
            this.errorCallbacks = [];
            this.clickCallbacks = [];
            this.loadResolves = [];
            this.loadRejects = [];
            this.showResolves = [];
            this.showRejects = [];
        }

        setCustomData(object) {
            try {
                let customData = '';
                if (null != object && '' != object) {
                    customData = JSON.stringify(object);
                }
                jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', `set${this.typeName}CustomData`, '(Ljava/lang/String;Ljava/lang/String;)V', this.adUnitId, customData);
            } catch (e) {

            }
        }

        onLoad(callback) {
            if (isFunction(callback)) {
                this.loadCallbacks.push(callback);
            } else {
                console.error('参数必须是一个函数');
            }
        }

        offLoad(callback) {
            let idx = this.loadCallbacks.indexOf(callback);
            if (idx === -1) {
                console.warn('没有注册该回调');
            } else {
                this.loadCallbacks.splice(idx, 1);
            }
        }

        onError(callback) {
            if (isFunction(callback)) {
                this.errorCallbacks.push(callback);
            } else {
                console.error('参数必须是一个函数');
            }
        }

        offError(callback) {
            let idx = this.errorCallbacks.indexOf(callback);
            if (idx === -1) {
                console.warn('没有注册该回调');
            } else {
                this.errorCallbacks.splice(idx, 1);
            }
        }

        onClick(callback) {
            if (isFunction(callback)) {
                this.clickCallbacks.push(callback);
            } else {
                console.error('参数必须是一个函数');
            }
        }

        offClick(callback) {
            let idx = this.clickCallbacks.indexOf(callback);
            if (idx === -1) {
                console.warn('没有注册该回调');
            } else {
                this.clickCallbacks.splice(idx, 1);
            }
        }
    }

    class BaseViewAd extends BaseAd {
        constructor(adUnitId, style, typeName) {
            super();
            this.adUnitId = adUnitId;
            this.typeName = typeName;
            this.resizeCallbacks = [];
            this.style = new AdStyle(adUnitId, style, typeName);
        }

        onResize(callback) {
            if (isFunction(callback)) {
                this.resizeCallbacks.push(callback);
            } else {
                console.error('参数必须是一个函数');
            }
        }

        offResize(callback) {
            let idx = this.resizeCallbacks.indexOf(callback);
            if (idx === -1) {
                console.warn('没有注册该回调');
            } else {
                this.resizeCallbacks.splice(idx, 1);
            }
        }

        show() {
            return new Promise((resolve, reject) => {
                this.showResolves.push(resolve);
                this.showRejects.push(reject);
                try {
                    jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', `show${this.typeName}`, '(Ljava/lang/String;)V', this.adUnitId);
                } catch (e) {
                    this.showResolves.length = 0;
                    this.showRejects.length = 0;
                    setTimeout(() => {
                        reject();
                    });
                }
            });
        }

        hide() {
            this.showResolves.length = 0;
            this.showRejects.length = 0;
            try {
                jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', `hide${this.typeName}`, '(Ljava/lang/String;)V', this.adUnitId);
            } catch (e) {
            }
        }

        destroy() {
            try {
                jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', `destroy${this.typeName}`, '(Ljava/lang/String;)V', this.adUnitId);
            } catch (e) {
            }
        }
    }

    class BaseVideoAd extends BaseAd {
        constructor(adUnitId, typeName) {
            super();
            this.adUnitId = adUnitId;
            this.typeName = typeName;
            this.closeCallbacks = [];
        }

        load() {
            return new Promise((resolve, reject) => {
                this.loadResolves.push(resolve);
                this.loadRejects.push(reject);
                if (this.loadResolves.length > 1) { // 正在加载中
                    return;
                }
                try {
                    jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', `load${this.typeName}`, '(Ljava/lang/String;)V', this.adUnitId);
                } catch (e) {
                    this.loadResolves.length = 0;
                    this.loadRejects.length = 0;
                    setTimeout(() => {
                        reject();
                    });
                }
            });
        }



        show(object) {
            return new Promise((resolve, reject) => {
                this.showResolves.push(resolve);
                this.showRejects.push(reject);
                if (this.showResolves.length > 1) { // 正在显示中
                    return;
                }
                let sceneId = '';
                if (null != object) {
                    if (typeof object == 'object') {
                        sceneId = object.sceneId || object.name || '';
                    } else {
                        sceneId = object;
                    }
                }
                try {
                    jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', `show${this.typeName}`, '(Ljava/lang/String;Ljava/lang/String;)V', this.adUnitId, sceneId);
                } catch (e) {
                    this.showResolves.length = 0;
                    this.showRejects.length = 0;
                    setTimeout(() => {
                        reject();
                    });
                }
            });
        }

        onClose(callback) {
            if (typeof callback === 'function') {
                this.closeCallbacks.push(callback);
            } else {
                console.error('参数必须是一个函数');
            }
        }

        offClose(callback) {
            let idx = this.closeCallbacks.indexOf(callback);
            if (idx === -1) {
                console.warn('没有注册该回调');
            } else {
                this.closeCallbacks.splice(idx, 1);
            }
        }

        destroy() {
            try {
                jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', `destroy${this.typeName}`, '(Ljava/lang/String;)V', this.adUnitId);
            } catch (e) {
            }
        }
    }

    class BannerAd extends BaseViewAd {
        constructor(adUnitId, style) {
            super(adUnitId, style, 'BannerAd');
        }
    }

    class NativeAd extends BaseViewAd {
        constructor(adUnitId, style) {
            super(adUnitId, style, 'NativeAd');
        }
    }

    class InterstitialAd extends BaseVideoAd {
        constructor(adUnitId) {
            super(adUnitId, 'InterstitialAd');
        }
    }

    class RewardedVideoAd extends BaseVideoAd {
        constructor(adUnitId) {
            super(adUnitId, 'RewardedVideoAd');
        }
    }

    class BaseDP {
        constructor(adUnitId, typeName) {
            this.adUnitId = adUnitId;
            this.typeName = typeName;
        }

        hide() {
            try {
                jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', `hide${this.typeName}`, '(Ljava/lang/String;)V', this.adUnitId);
            } catch (e) {
            }
        }
    }

    class VideoDP extends BaseDP {

        constructor(adUnitId) {
            super(adUnitId, 'VideoDP');
        }

        show() {
            try {
                jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', `show${this.typeName}`, '(Ljava/lang/String;)V', this.adUnitId);
            } catch (e) {
            }
        }

        onVideoPlay(callback) {
            this.videoPlayCallback = callback;
        }

        onVideoOver(callback) {
            this.videoOverCallback = callback;
        }

        onVideoCompletion(callback) {
            this.videoCompletionCallback = callback;
        }

        onClickAuthorName(callback) {
            this.clickAuthorNameCallback = callback;
        }

        onClickAvatar(callback) {
            this.clickAvatarCallback = callback;
        }

        onClickComment(callback) {
            this.clickCommentCallback = callback;
        }

        onClickLike(callback) {
            this.clickLikeCallback = callback;
        }

        onAdShow(callback) {
            this.adShowCallback = callback;
        }

        onAdPlayStart(callback) {
            this.adPlayStartCallback = callback;
        }

        onAdPlayComplete(callback) {
            this.adPlayCompleteCallback = callback;
        }

        onAdClicked(callback) {
            this.adClickedCallback = callback;
        }
    }

    class ViewDP extends BaseDP {

        constructor(adUnitId) {
            super(adUnitId, 'ViewDP');
        }

        show(id) {
            try {
                jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', `show${this.typeName}`, '(Ljava/lang/String;I)V', this.adUnitId, id);
            } catch (e) {
            }
        }

        onLoadError(callback) {
            this.loadErrorCallback = callback;
        }

        onAdClick(callback) {
            this.adClickCallback = callback;
        }

        onAdImpression(callback) {
            this.adImpressionCallback = callback;
        }

        onContentClick(callback) {
            this.contentClickCallback = callback;
        }

        onContentImpression(callback) {
            this.contentImpressionCallback = callback;
        }
    }

    function isFunction(func) {
        return typeof func == 'function';
    }

    window.zs = window.zs || {};

    let bannerAds = {};
    let interstitialAds = {};
    let rewardedVideoAds = {};
    let rewardedVideoUserIds = {};
    let nativeAds = {};

    let _videoDPs = {};
    let _viewDPs = {};

    // 设备参数
    let deviceSessionResults = {};

    let authorizeResult = null;
    let policyResults = [];
    let appLoadResults = [];
    let clipboardCallbacks = [];
    let sdkLoginResults = [];

    let paySuccess = null;
    let payFail = null;
    let payComplete = null;

    let locationCallback = null;

    // 运行时支持
    zs.SUPPORT_RUNTIME = true

    zs.isDebug = function () {
        try {
            return jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'isDebug', '()Z');
        } catch (e) {
            return false;
        }
    }

    zs.getChannel = function () {
        try {
            return jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'getChannel', '()Ljava/lang/String;');
        } catch (e) {
            return "";
        }
    }

    zs.getAppName = function () {
        try {
            return jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'getAppName', '()Ljava/lang/String;');
        } catch (e) {
            return "";
        }
    }

    zs.getPackageName = function () {
        try {
            return jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'getPackageName', '()Ljava/lang/String;');
        } catch (e) {
            return "";
        }
    }

    zs.updateApp = function (object) {
        if (null == object) {
            return;
        }
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'updateApp', '(Ljava/lang/String;Ljava/lang/String;ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;Z)V', object.name, object.url, object.versionCode, object.versionName, object.size, object.des, object.force);
        } catch (e) {
        }
    }

    zs.getDeviceSession = function (object) {
        if (null == object) {
            return;
        }
        let type = object.type;
        if (null == type || '' == type) {
            type = 'ali';
        }
        if (null == deviceSessionResults[type]) {
            deviceSessionResults[type] = [];
        }
        deviceSessionResults[type].push(object);
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', "onDeviceSession", "(Ljava/lang/String;)V", type);
        } catch (e) { }
    }

    // 废弃
    zs.onDeviceSession = function (callback) {
        if (!isFunction(callback)) {
            console.error('参数不是一个方法');
            return;
        }
        zs.getDeviceSession({
            type: 'ali', success: (res) => {
                callback({ code: 0, session: res.session });
            }, fail: (res) => {
                callback({ code: res.errCode, session: res.errMsg });
            }
        });
    }

    // 废弃
    zs.offDeviceSession = function (callback) { }

    zs.getSystemInfoSync = function () {
        try {
            let info = jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'getSystemInfoBase64', '()Ljava/lang/String;');
            return JSON.parse(decodeURI(atob(info)));
        } catch (e) {
            return {}
        }
    }

    zs.isInstall = function (name) {
        if (null == name || "" == name) {
            return false;
        }
        try {
            return jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'isInstall', '(Ljava/lang/String;)Z', name);
        } catch (e) {
            return false;
        }
    }

    zs.setClipboardData = function (object) {
        if (null == object || null == object.data) {
            object && isFunction(object.fail) && object.fail();
        } else {
            try {
                jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'setClipboardData', '(Ljava/lang/String;)V', object.data);
                object && isFunction(object.success) && object.success();
            } catch (e) {
                object && isFunction(object.fail) && object.fail();
            }
        }
        object && isFunction(object.complete) && object.complete();
    }

    zs.getClipboardData = function (object) {
        if (null == object || !isFunction(object.success)) {
            console.error('回调方法不存在');
            return;
        }
        if (null != object) {
            if (clipboardCallbacks.length > 0) {
                clipboardCallbacks.push(object);
                return;
            }
            try {
                clipboardCallbacks.push(object);
                jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'getClipboardData', '()V');
            } catch (e) {
                clipboardCallbacks.splice(clipboardCallbacks.length - 1, 1);
                object && isFunction(object.fail) && object.fail();
                object && isFunction(object.complete) && object.complete();
            }
        }
    }

    zs.request = function (object) {
        try {
            let data = '';
            if (null != object.data) {
                data = JSON.stringify(object.data);
            }
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'request', '(Ljava/lang/String;Ljava/lang/String;)V', object.url, data);
        } catch (e) {
            object && isFunction(object.fail) && object.fail();
            object && isFunction(object.complete) && object.complete();
        }
    }

    zs.vibrateShort = function (object) {
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'vibrate', '(I)V', 15);
            object && isFunction(object.success) && object.success();
        } catch (e) {
            object && isFunction(object.fail) && object.fail();
        }
        object && isFunction(object.complete) && object.complete();
    }

    zs.vibrateLong = function (object) {
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'vibrate', '(I)V', 400);
            object && isFunction(object.success) && object.success();
        } catch (e) {
            object && isFunction(object.fail) && object.fail();
        }
        object && isFunction(object.complete) && object.complete();
    }

    zs.getLocation = function (object) {
        if (null == object) {
            return;
        }
        locationCallback = object.success;
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'getLocation', '()V');
        } catch (e) {
            object && isFunction(object.fail) && object.fail();
        }
    }

    zs.onPolicyResult = function (callback) {
        policyResults.push(callback);
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', "onPolicyResult", "()V");
        } catch (e) { }
    }

    zs.offPolicyResult = function (callback) {
        let idx = policyResults.indexOf(callback);
        if (idx === -1) {
            console.warn('没有注册该回调');
        } else {
            policyResults.splice(idx, 1);
        }
    }

    zs.gameLoadResult = function () {
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', "gameLoadResult", "()V");
        } catch (e) { }
    }

    zs.onAppLoadResult = function (callback) {
        if (!isFunction(callback)) {
            console.error('参数不是一个方法');
            return;
        }
        appLoadResults.push(callback);
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', "onAppLoadResult", "()V");
        } catch (e) { }
    }

    zs.offAppLoadResult = function (callback) {
        let idx = appLoadResults.indexOf(callback);
        if (idx === -1) {
            console.warn('没有注册该回调');
        } else {
            appLoadResults.splice(idx, 1);
        }
    }

    zs.onSdkLoginResult = function (callback) {
        if (!isFunction(callback)) {
            console.error('参数不是一个方法');
            return;
        }
        sdkLoginResults.push(callback);
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'onSdkLoginResult', '()V');
        } catch (e) { }
    }

    zs.offSdkLoginResult = function (callback) {
        let idx = sdkLoginResults.indexOf(callback);
        if (idx === -1) {
            console.warn('没有注册该回调');
        } else {
            sdkLoginResults.splice(idx, 1);
        }
    }

    zs.reportEvent = function (object) {
        if (null == object) {
            return;
        }
        try {
            let json = JSON.stringify(object);
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'reportEvent', '(Ljava/lang/String;)V', json);
        } catch (e) {
        }
    }

    zs.activeEvent = function () {
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'activeEvent', '()V');
        } catch (e) { }
    }

    zs.registerEvent = function (object) {
        if (null == object) {
            object = {};
        }
        if (null == object.isSuccess) {
            object.isSuccess = true;
        }
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'registerEvent', '(Ljava/lang/String;Ljava/lang/String;Z)V', object.pf || 'zs', object.userId || '', object.isSuccess);
        } catch (e) { }
    }

    zs.loginEvent = function (object) {
        if (null == object) {
            object = {};
        }
        if (null == object.isSuccess) {
            object.isSuccess = true;
        }
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'loginEvent', '(Ljava/lang/String;Ljava/lang/String;Z)V', object.pf || 'zs', object.userId || '', object.isSuccess);
        } catch (e) { }
    }

    zs.nextDayStayEvent = function () {
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'nextDayStayEvent', '()V');
        } catch (e) { }
    }

    zs.weekStayEvent = function () {
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'weekStayEvent', '()V');
        } catch (e) { }
    }

    zs.payEvent = function (object) {
        if (null == object) {
            object = {};
        }
        if (null == object.isSuccess) {
            object.isSuccess = true;
        }
        if (null == object.virtual) {
            object.virtual = false;
        }
        try {
            jsb.reflection.callStaticMethod(
                'com/zhise/bridge/ZBSdk',
                'payEvent',
                '(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;ILjava/lang/String;Ljava/lang/String;ZIZ)V',
                object.userId || '',
                object.contentType || 'type',
                object.contentName || 'name',
                object.contentId || 'id',
                object.contentNumber || 1,
                object.paymentChannel || 'zs',
                object.currency || 'RMB',
                object.isSuccess || true,
                object.currencyAmount || 1,
                object.virtual
            );
        } catch (e) { }
    }

    zs.keyEvent = function (object) {
        if (null == object) {
            return;
        }
        if (null == object.eventId) {
            return;
        }
        let params;
        if (null == object.params) {
            params = '';
        } else {
            try {
                params = JSON.stringify(object.params);
            } catch (e) { }
        }
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'keyEvent', '(Ljava/lang/String;Ljava/lang/String;)V', object.eventId, params);
        } catch (e) { }
    }

    // 废弃
    zs.customEvent = zs.keyEvent;

    zs.onEvent = function (eventId, params) {
        try {
            let str = '';
            if (null != params) {
                str = JSON.stringify(params);
            }
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'onEvent', '(Ljava/lang/String;Ljava/lang/String;)V', eventId, str);
        } catch (e) { }
    }

    zs.onCustomEvent = function (object) {
        if (null == object) {
            return;
        }
        try {
            let str = '';
            if (null != object) {
                str = JSON.stringify(object);
            }
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'onCustomEvent', '(Ljava/lang/String;)V', str);
        } catch (e) { }
    }

    zs.reportError = function (object) {
        if (null == object) {
            return;
        }
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'reportError', '(Ljava/lang/String;Ljava/lang/String;)V', object.title, object.info);
        } catch (e) {
        }
    }

    zs.putBuglyUserData = function (key, value) {
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'putBuglyUserData', '(Ljava/lang/String;Ljava/lang/String;)V', key, value);
        } catch (e) { }
    }

    zs.onEventBegin = function (eventId, params) {
        try {
            let str = '';
            if (null != params) {
                str = JSON.stringify(params);
            }
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'onEventBegin', '(Ljava/lang/String;Ljava/lang/String;)V', eventId, str);
        } catch (e) { }
    }

    zs.onEventEnd = function (eventId, params) {
        try {
            let str = '';
            if (null != params) {
                str = JSON.stringify(params);
            }
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'onEventEnd', '(Ljava/lang/String;Ljava/lang/String;)V', eventId, str);
        } catch (e) { }
    }

    zs.onPageStart = function (pageName) {
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'onPageStart', '(Ljava/lang/String;)V', pageName);
        } catch (e) { }
    }

    zs.onPageEnd = function (pageName) {
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'onPageEnd', '(Ljava/lang/String;)V', pageName);
        } catch (e) { }
    }

    zs.createBannerAd = function (object) {
        if (null == object) {
            return null;
        }
        if (null == object.style) {
            return null;
        }
        var adUnitId = object.adUnitId;
        if (null == adUnitId) {
            adUnitId = '';
        }
        if (bannerAds[adUnitId]) {
            return bannerAds[adUnitId];
        }
        if (null == object.adIntervals) {
            object.adIntervals = 30;
        }
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'createBannerAd', '(Ljava/lang/String;IIIII)V', adUnitId, object.adIntervals, object.style.left, object.style.top, object.style.width, object.style.height);
            bannerAds[adUnitId] = new BannerAd(adUnitId, object.style);
            return bannerAds[adUnitId];
        } catch (e) {
            return null;
        }
    }

    zs.createInterstitialAd = function (object) {
        if (null == object) {
            object = {};
        }
        let adUnitId = object.adUnitId;
        if (null == adUnitId) {
            return null;
        }
        if (null != interstitialAds[adUnitId]) {
            return interstitialAds[adUnitId];
        }
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'createInterstitialAd', '(Ljava/lang/String;)V', adUnitId);
            interstitialAds[adUnitId] = new InterstitialAd(adUnitId);
            return interstitialAds[adUnitId];
        } catch (e) {
            return null;
        }
    }

    zs.createRewardedVideoAd = function (object) {
        if (null == object) {
            object = {};
        }
        let adUnitId = object.adUnitId;
        if (null == adUnitId) {
            return null;
        }
        let userId;
        if (null == object.userId) {
            userId = '';
        } else {
            userId = object.userId + '';
        }
        if (null != rewardedVideoAds[adUnitId] && userId == rewardedVideoUserIds[adUnitId]) {
            return rewardedVideoAds[adUnitId];
        }
        rewardedVideoUserIds[adUnitId] = userId;
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'createRewardedVideoAd', '(Ljava/lang/String;Ljava/lang/String;)V', adUnitId, userId);
            rewardedVideoAds[adUnitId] = new RewardedVideoAd(adUnitId);
            return rewardedVideoAds[adUnitId];
        } catch (e) {
            return null;
        }
    }

    zs.createNativeAd = function (object) {
        if (null == object) {
            return null;
        }
        if (null == object.style) {
            return null;
        }
        var adUnitId = object.adUnitId;
        if (null == adUnitId) {
            adUnitId = '';
        }
        if (nativeAds[adUnitId]) {
            return nativeAds[adUnitId];
        }
        try {
            console.log('ZS_SDK createNativeAd', JSON.stringify(object.style));
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'createNativeAd', '(Ljava/lang/String;IIII)V', object.adUnitId, object.style.left, object.style.top, object.style.width, object.style.height);
            nativeAds[adUnitId] = new NativeAd(adUnitId, object.style);
            return nativeAds[adUnitId];
        } catch (e) {
            return null;
        }
    }

    zs.createVideoDP = function (object) {
        if (null == object) {
            return null;
        }
        if (null == object.style) {
            return null;
        }
        if (null == object.adUnitId) {
            object.adUnitId = '';
        }
        if (null != _videoDPs[object.adUnitId]) {
            return _videoDPs[object.adUnitId];
        }
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'createVideoDP', '(Ljava/lang/String;IIII)V', object.adUnitId, object.style.left, object.style.top, object.style.width, object.style.height);
            _videoDPs[object.adUnitId] = new VideoDP(object.adUnitId, object.style);
            return _videoDPs[object.adUnitId];
        } catch (e) {
            return null;
        }
    }

    zs.createViewDP = function (object) {
        if (null == object) {
            return null;
        }
        if (null == object.style) {
            return null;
        }
        if (null == object.adUnitId) {
            object.adUnitId = '';
        }
        if (null != _viewDPs[object.adUnitId]) {
            return _viewDPs[object.adUnitId];
        }
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'createViewDP', '(Ljava/lang/String;IIII)V', object.adUnitId, object.style.left, object.style.top, object.style.width, object.style.height);
            _viewDPs[object.adUnitId] = new ViewDP(object.adUnitId, object.style);
            return _viewDPs[object.adUnitId];
        } catch (e) {
            return null;
        }
    }

    zs.showToast = function (object) {
        if (null == object) {
            return;
        }
        if (null == object.title) {
            return;
        }
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'showToast', '(Ljava/lang/String;)V', object.title);
        } catch (e) { }
    }

    zs.login = function (object) {
        if (null == object) {
            return;
        }
        try {
            const did = jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'login', '()Ljava/lang/String;');
            object.success && object.success({ code: did });
        } catch (e) {
            object.fail && object.fail();

        }
        object.complete && object.complete();
    }

    zs.logout = function () {
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'logout', '()V');
        } catch (e) {
        }
    }

    zs.authorize = function (object) {
        authorizeResult = { success: object.success, fail: object.fail, complete: object.complete };
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'authorize', '(Ljava/lang/String;)V', object.scope || 'wx');
        } catch (e) {
            object && object.fail && object.fail();
            object && object.complete && object.complete();
        }
    }

    zs.shareAppMessage = function (object) {

    }

    zs.showNotification = function (object) {
        try {
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'showNotification', '(ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;III)V', object.notificationId || 0, object.title || '', object.content || '', object.buttonContent || '', object.jumpActivity || '', object.visibility || 1, object.priority || 2, object.importance || 3);
        } catch (e) {
        }
    }

    /**
     * 支付
     */
    zs.pay = function (object) {
        try {
            paySuccess = object.success;
            payFail = object.fail;
            payComplete = object.complete;
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'pay',
                '(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V',
                object.payType || 'wx',
                object.appId,
                object.partnerId,
                object.prepayId,
                object.packageValue || 'Sign=WXPay',
                object.nonceStr,
                object.timeStamp,
                object.sign
            );
        } catch (e) {
            object.fail && object.fail();
            object.complete && object.complete();
        }
    }

    window.zsCall = {};

    zsCall.onDeviceSession = function (type, code, msg) {
        if (null == deviceSessionResults[type]) {
            return;
        }
        msg = atob(msg);
        deviceSessionResults[type].forEach((object) => {
            if (0 == code) {
                object.success && object.success({ session: msg });
            } else {
                object.fail && object.fail({ errCode: code, errMsg: msg });
            }
            object.complete && object.complete();
        });
        deviceSessionResults[type].length = 0;
    }

    zsCall.onClipboardData = function (text) {
        clipboardCallbacks.forEach((object) => {
            object && isFunction(object.success) && object.success({ data: text });
        });
        clipboardCallbacks.length = 0;
    }

    zsCall.onPolicyResult = function (bool) {
        setTimeout(() => {
            policyResults.forEach((callback) => {
                callback && callback(bool);
            });
        }, 500);

    }

    zsCall.onAppLoadResult = function () {
        appLoadResults.forEach((object) => {
            object();
        });
    }

    zsCall.onSdkLoginResult = function (uuid, attribute) {
        sdkLoginResults.forEach((object) => {
            object({ uuid, attribute });
        });
    }

    zsCall.onAuthor = function (errCode, msg) {
        setTimeout(() => {
            if (0 == errCode) {
                msg = atob(msg);
                authorizeResult && isFunction(authorizeResult.success) && authorizeResult.success({ errCode: 0, code: msg });
            } else {
                authorizeResult && isFunction(authorizeResult.fail) && authorizeResult.fail({ errCode: errCode, errMsg: msg });
            }
            authorizeResult && isFunction(authorizeResult.complete) && authorizeResult.complete();
            authorizeResult = null;
        }, 150);
    }

    function onAdResize(ad, width, height) {
        if (null != ad) {
            ad.resizeCallbacks.forEach((callback) => {
                callback({ width, height });
            });
        }
    }

    function onAdLoad(ad) {
        if (null != ad) {
            ad.loadCallbacks.forEach((callback) => {
                callback();
            });
        }
    }

    function onAdError(ad, errCode, errMsg) {
        setTimeout(() => {
            if (null != ad) {
                ad.errorCallbacks.forEach((callback) => {
                    callback(errCode, errMsg);
                });
            }
        }, 150);
    }

    function adLoadResolve(ad) {
        if (null != ad) {
            let loadResolves = ad.loadResolves;
            ad.loadResolves = [];
            ad.loadRejects.length = 0;
            loadResolves.forEach((resolve) => {
                resolve();
            });
            loadResolves = null;
        }
    }

    function adLoadReject(ad) {
        setTimeout(() => {
            if (null != ad) {
                let loadRejects = ad.loadRejects;
                ad.loadRejects = [];
                ad.loadResolves.length = 0;
                loadRejects.forEach((reject) => {
                    reject();
                });
                loadRejects = null;
            }
        });
    }

    function adShowResolve(ad, ecpm) {
        if (null != ad) {
            let showResolves = ad.showResolves;
            ad.showResolves = [];
            ad.showRejects.length = 0;
            showResolves.forEach((resolve) => {
                resolve({ ecpm });
            });
            showResolves = null;
        }
    }

    function adShowReject(ad) {
        setTimeout(() => {
            if (null != ad) {
                let showRejects = ad.showRejects;
                ad.showRejects = [];
                ad.showResolves.length = 0;
                showRejects.forEach((reject) => {
                    reject();
                });
                showRejects = null;
            }
        }, 150);
    }

    function onAdClick(ad) {
        setTimeout(() => {
            if (null != ad) {
                ad.clickCallbacks.forEach((callback) => {
                    callback();
                });
            }
        }, 150);
    }

    zsCall.onBannerAdResize = function (adUnitId, width, height) {
        onAdResize(bannerAds[adUnitId], width, height);
    }

    zsCall.onBannerAdLoad = function (adUnitId) {
        onAdLoad(bannerAds[adUnitId]);
    }

    zsCall.onBannerAdLoadError = function (adUnitId, errCode, errMsg) {
        onAdError(bannerAds[adUnitId], errCode, errMsg);
    }

    zsCall.onBannerAdShow = function (adUnitId) {
    }

    zsCall.onBannerAdShowError = function (adUnitId, errCode, errMsg) {
        onAdError(bannerAds[adUnitId], errCode, errMsg);
    }

    zsCall.onInterstitialAdClose = function (adUnitId) {
        setTimeout(() => {
            let ad = interstitialAds[adUnitId];
            if (null != ad) {
                ad.closeCallbacks.forEach((callback) => {
                    callback({});
                });
            }
        }, 150);
    }

    zsCall.onInterstitialAdLoad = function (adUnitId) {
        onAdLoad(interstitialAds[adUnitId]);
        adLoadResolve(interstitialAds[adUnitId]);
    }

    zsCall.onInterstitialAdLoadError = function (adUnitId, errCode, errMsg) {
        onAdError(interstitialAds[adUnitId], errCode, errMsg);
        adLoadReject(interstitialAds[adUnitId]);
    }

    zsCall.onInterstitialAdShow = function (adUnitId) {
        adShowResolve(interstitialAds[adUnitId]);
    }

    zsCall.onInterstitialAdShowError = function (adUnitId, errCode, errMsg) {
        onAdError(interstitialAds[adUnitId], errCode, errMsg);
        adShowReject(interstitialAds[adUnitId]);
    }

    zsCall.onInterstitialAdClick = function (adUnitId) {
        onAdClick(interstitialAds[adUnitId]);
    }

    zsCall.onRewardedVideoAdClose = function (adUnitId, verify, ecpm) {
        setTimeout(() => {
            let ad = rewardedVideoAds[adUnitId];
            if (null != ad) {
                ad.closeCallbacks.forEach((callback) => {
                    callback({ isEnded: verify, ecpm: ecpm });
                });
            }
        }, 150);
    }

    zsCall.onRewardedVideoAdLoad = function (adUnitId) {
        onAdLoad(rewardedVideoAds[adUnitId]);
        adLoadResolve(rewardedVideoAds[adUnitId]);
    }

    zsCall.onRewardedVideoAdLoadError = function (adUnitId, errCode, errMsg) {
        onAdError(rewardedVideoAds[adUnitId], errCode, errMsg);
        adLoadReject(rewardedVideoAds[adUnitId]);
    }

    zsCall.onRewardedVideoAdShow = function (adUnitId, ecpm) {
        adShowResolve(rewardedVideoAds[adUnitId], ecpm);
    }

    zsCall.onRewardedVideoAdShowError = function (adUnitId, errCode, errMsg) {
        onAdError(rewardedVideoAds[adUnitId], errCode, errMsg);
        adShowReject(rewardedVideoAds[adUnitId]);
    }

    zsCall.onRewardedVideoAdClick = function (adUnitId) {
        onAdClick(rewardedVideoAds[adUnitId]);
    }

    zsCall.onNativeAdResize = function (adUnitId, width, height) {
        onAdResize(nativeAds[adUnitId], width, height);
    }

    zsCall.onNativeAdLoad = function (adUnitId) {
        onAdLoad(nativeAds[adUnitId]);
    }

    zsCall.onNativeAdLoadError = function (adUnitId, errCode, errMsg) {
        onAdError(nativeAds[adUnitId], errCode, errMsg);
    }

    zsCall.onNativeAdShow = function (adUnitId) {
    }

    zsCall.onNativeAdShowError = function (adUnitId, errCode, errMsg) {
        onAdError(nativeAds[adUnitId], errCode, errMsg);
    }

    zsCall.onVideoDPVideoPlay = function (adUnitId) {
        if (null != _videoDPs[adUnitId] && null != _videoDPs[adUnitId].videoPlayCallback) {
            _videoDPs[adUnitId].videoPlayCallback();
        }
    }

    zsCall.onVideoDPVideoOver = function (adUnitId) {
        if (null != _videoDPs[adUnitId] && null != _videoDPs[adUnitId].videoOverCallback) {
            _videoDPs[adUnitId].videoOverCallback();
        }
    }

    zsCall.onVideoDPVideoCompletion = function (adUnitId) {
        if (null != _videoDPs[adUnitId] && null != _videoDPs[adUnitId].videoCompletionCallback) {
            _videoDPs[adUnitId].videoCompletionCallback();
        }
    }

    zsCall.onVideoDPClickAuthorName = function (adUnitId) {
        if (null != _videoDPs[adUnitId] && null != _videoDPs[adUnitId].clickAuthorNameCallback) {
            _videoDPs[adUnitId].clickAuthorNameCallback();
        }
    }

    zsCall.onVideoDPClickAvatar = function (adUnitId) {
        if (null != _videoDPs[adUnitId] && null != _videoDPs[adUnitId].clickAvatarCallback) {
            _videoDPs[adUnitId].clickAvatarCallback();
        }
    }

    zsCall.onVideoDPClickComment = function (adUnitId) {
        if (null != _videoDPs[adUnitId] && null != _videoDPs[adUnitId].clickCommentCallback) {
            _videoDPs[adUnitId].clickCommentCallback();
        }
    }

    zsCall.onVideoDPClickLike = function (adUnitId) {
        if (null != _videoDPs[adUnitId] && null != _videoDPs[adUnitId].clickLikeCallback) {
            _videoDPs[adUnitId].clickLikeCallback();
        }
    }

    zsCall.onVideoDPAdShow = function (adUnitId) {
        if (null != _videoDPs[adUnitId] && null != _videoDPs[adUnitId].adShowCallback) {
            _videoDPs[adUnitId].adShowCallback();
        }
    }

    zsCall.onVideoDPAdPlayStart = function (adUnitId) {
        if (null != _videoDPs[adUnitId] && null != _videoDPs[adUnitId].adPlayStartCallback) {
            _videoDPs[adUnitId].adPlayStartCallback();
        }
    }

    zsCall.onVideoDPAdPlayComplete = function (adUnitId) {
        if (null != _videoDPs[adUnitId] && null != _videoDPs[adUnitId].adPlayCompleteCallback) {
            _videoDPs[adUnitId].adPlayCompleteCallback();
        }
    }

    zsCall.onVideoDPAdClicked = function (adUnitId) {
        if (null != _videoDPs[adUnitId] && null != _videoDPs[adUnitId].adClickedCallback) {
            _videoDPs[adUnitId].adClickedCallback();
        }
    }

    zsCall.onViewDPLoadError = function (adUnitId, code, msg) {
        if (null != _viewDPs[adUnitId] && null != _viewDPs[adUnitId].loadErrorCallback) {
            _viewDPs[adUnitId].loadErrorCallback({
                errorCode: code,
                errorMsg: msg
            });
        }
    }

    zsCall.onViewDPAdClick = function (adUnitId) {
        if (null != _viewDPs[adUnitId] && null != _viewDPs[adUnitId].adClickCallback) {
            _viewDPs[adUnitId].adClickCallback();
        }
    }

    zsCall.onViewDPAdImpression = function (adUnitId, num) {
        if (null != _viewDPs[adUnitId] && null != _viewDPs[adUnitId].adImpressionCallback) {
            _viewDPs[adUnitId].adImpressionCallback({ num });
        }
    }

    zsCall.onViewDPContentClick = function (adUnitId) {
        if (null != _viewDPs[adUnitId] && null != _viewDPs[adUnitId].contentClickCallback) {
            _viewDPs[adUnitId].contentClickCallback();
        }
    }

    zsCall.onViewDPContentImpression = function (adUnitId, num) {
        if (null != _viewDPs[adUnitId] && null != _viewDPs[adUnitId].contentImpressionCallback) {
            _viewDPs[adUnitId].contentImpressionCallback({ num });
        }
    }

    zsCall.onLocation = function (latitude, longitude, city, subCity, addressLine) {
        if (null != locationCallback) {
            locationCallback({
                latitude, longitude, city, subCity, addressLine
            });
        }
    }

    zsCall.onPayResult = function (errCode, errMsg) {
        if (errCode == 0) {
            paySuccess && paySuccess();
        } else {
            payFail && payFail(errCode, errMsg);
        }
        payComplete && payComplete();
    }

})();