(function () {

    // 创建脚本代理
    let bridge = PlatformClass.createClass('com.zhise.bridge.ZBSdk');

    class AdStyle {

        /**
         * 
         * @param {Number} adId 
         * @param {any} style 
         * @param {String} typeName
         */
        constructor(adId, style, typeName) {
            this.adId = adId;
            this._width = style.width;
            this._height = style.height;
            this._left = style.left;
            this._top = style.top;
            this.typeName = typeName;
        }

        /**
         * 
         */
        get width() {
            return this._width;
        }

        /**
         * 
         */
        get height() {
            return this._height;
        }

        /**
         * 
         */
        set left(left) {
            this._left = left;
            try {
                bridge.call(`set${this.typeName}Left`, this.adId, left);
            } catch (e) { }
        }

        /**
         * 
         */
        get left() {
            return this._left
        }

        /**
         * 
         */
        set top(top) {
            this._top = top;
            try {
                bridge.call(`set${this.typeName}Top`, this.adId, top);
            } catch (e) { }
        }

        /**
         * 
         */
        get top() {
            return this._top;
        }
    }

    /**
     * BaseAd
     */
    class BaseAd {

        constructor() {
            this.loadCallbacks = [];
            this.errorCallbacks = [];
            this.clickCallbacks = [];
            this.loadResolves = [];
            this.loadRejects = [];
            this.showResolves = [];
            this.showRejects = [];

        }

        setCustomData(object) {
            try {
                let customData = '';
                if (null != object && '' != object) {
                    customData = JSON.stringify(object);
                }
                bridge.call(`set${this.typeName}CustomData`, this.adUnitId, customData);
            } catch (e) {
            }
        }

        /**
         * 
         * @param {Function} callback 
         */
        onLoad(callback) {
            if (typeof callback === "function") {
                this.loadCallbacks.push(callback);
            } else {
                console.error('参数必须是一个函数');
            }
        }

        /**
         * 
         * @param {Function} callback 
         */
        offLoad(callback) {
            let idx = this.loadCallbacks.indexOf(callback);
            if (idx === -1) {
                console.warn('没有注册该回调');
            } else {
                this.loadCallbacks.splice(idx, 1);
            }
        }

        /**
         *
         * @param {Function} callback
         */
        onError(callback) {
            if (typeof callback === 'function') {
                this.errorCallbacks.push(callback);
            } else {
                console.error('参数必须是一个函数');
            }
        }

        /**
         * 
         * @param {Function} callback 
         */
        offError(callback) {
            let idx = this.errorCallbacks.indexOf(callback);
            if (idx === -1) {
                console.warn('没有注册该回调');
            } else {
                this.errorCallbacks.splice(idx, 1);
            }
        }

        onClick(callback) {
            if (typeof callback === 'function') {
                this.clickCallbacks.push(callback);
            } else {
                console.error('参数必须是一个函数');
            }
        }

        offClick(callback) {
            let idx = this.clickCallbacks.indexOf(callback);
            if (idx === -1) {
                console.warn('没有注册该回调');
            } else {
                this.clickCallbacks.splice(idx, 1);
            }
        }

    }

    class BaseViewAd extends BaseAd {
        constructor(adId, style, typeName) {
            super();
            this.adId = adId;
            this.typeName = typeName;
            this.resizeCallbacks = [];
            this.style = new AdStyle(adId, style, typeName);
        }

        /**
         *
         * @param {Function} callback
         */
        onResize(callback) {
            if (typeof callback === 'function') {
                this.resizeCallbacks.push(callback);
            } else {
                console.error('参数必须是一个函数');
            }
        }

        /**
         *
         * @param {Function} callback
         */
        offResize(callback) {
            let idx = this.resizeCallbacks.indexOf(callback);
            if (idx === -1) {
                console.warn('没有注册该回调');
            } else {
                this.resizeCallbacks.splice(idx, 1);
            }
        }

        /**
         *
         */
        show() {
            return new Promise((resolve, reject) => {
                this.showResolves.push(resolve);
                this.showRejects.push(reject);
                try {
                    bridge.call(`show${this.typeName}`, this.adId);
                } catch (e) {
                    this.showResolves.length = 0;
                    this.showRejects.length = 0;
                    setTimeout(() => {
                        reject();
                    });
                }
            });
        }

        /**
         *
         */
        hide() {
            this.showResolves.length = 0;
            this.showRejects.length = 0;
            try {
                bridge.call(`hide${this.typeName}`, this.adId);
            } catch (e) {
            }
        }

        /**
         *
         */
        destroy() {
            try {
                bridge.call(`destroy${this.typeName}`, this.adId);
            } catch (e) {
            }
        }
    }

    class BaseVideoAd extends BaseAd {
        constructor(adUnitId, typeName) {
            super();
            this.adUnitId = adUnitId;
            this.typeName = typeName;
            this.closeCallbacks = [];
        }

        load() {
            return new Promise((resolve, reject) => {
                this.loadResolves.push(resolve);
                this.loadRejects.push(reject);
                if (this.loadResolves.length > 1) {
                    return;
                }
                try {
                    bridge.call(`load${this.typeName}`, this.adUnitId);
                } catch (e) {
                    this.loadResolves.length = 0;
                    this.loadRejects.length = 0;
                    setTimeout(() => {
                        reject();
                    });
                }
            });
        }

        show(object) {
            return new Promise((resolve, reject) => {
                this.showResolves.push(resolve);
                this.showRejects.push(reject);
                if (this.showResolves.length > 1) {
                    return;
                }
                let sceneId = '';
                if (null != object) {
                    if (typeof object == 'object') {
                        sceneId = object.sceneId || object.name || '';
                    } else {
                        sceneId = object;
                    }
                }
                try {
                    bridge.call(`show${this.typeName}`, this.adUnitId, sceneId);
                } catch (e) {
                    this.showResolves.length = 0;
                    this.showRejects.length = 0;
                    setTimeout(() => {
                        reject();
                    });
                }
            });
        }

        /**
         *
         * @param {Function} callback
         */
        onClose(callback) {
            if (typeof callback === 'function') {
                this.closeCallbacks.push(callback);
            } else {
                console.error('参数必须是一个函数');
            }
        }

        /**
         *
         * @param {Function} callback
         */
        offClose(callback) {
            let idx = this.closeCallbacks.indexOf(callback);
            if (idx === -1) {
                console.warn('没有注册该回调');
            } else {
                this.closeCallbacks.splice(idx, 1);
            }
        }

        /**
         *
         */
        destroy() {
            try {
                bridge.call(`destroy${this.typeName}`, this.adUnitId);
            } catch (e) {
            }
        }
    }

    class BannerAd extends BaseViewAd {
        constructor(adId, style) {
            super(adId, style, 'BannerAd');
        }
    }

    class NativeAd extends BaseViewAd {
        constructor(adId, style) {
            super(adId, style, 'NativeAd');
        }
    }

    class InterstitialAd extends BaseVideoAd {
        constructor(adUnitId) {
            super(adUnitId, 'InterstitialAd');
        }
    }

    class RewardedVideoAd extends BaseVideoAd {
        constructor(adUnitId) {
            super(adUnitId, 'RewardedVideoAd');
        }
    }

    function isFunction(func) {
        return typeof func == 'function';
    }

    window.zs = window.zs || {};
    // window.wx = zs;
    let bannerAds = {};
    let interstitialAds = {};
    let rewardedVideoAds = {};
    let rewardedVideoUserIds = {};
    let nativeAds = {};

    let deviceSessionResults = {};

    let authorizeResult = null;
    let policyResults = [];
    let appLoadResults = [];
    let clipboardCallbacks = [];
    let sdkLoginResults = [];

    zs.isDebug = function () {
        try {
            return bridge.call(`isDebug`);
        } catch (e) {
            return false;
        }
    }

    zs.getChannel = function () {
        try {
            return bridge.call(`getChannel`);
        } catch (e) {
            return '';
        }
    }

    zs.getAppName = function () {
        try {
            return bridge.call(`getAppName`);
        } catch (e) {
            return '';
        }
    }

    zs.getPackageName = function () {
        try {
            return bridge.call(`getPackageName`);
        } catch (e) {
            return '';
        }
    }

    zs.updateApp = function (object) {
        if (null == object) {
            return;
        }
        try {
            bridge.call('updateApp', object.name, object.url, object.versionCode, object.versionName, object.size, object.des, object.force);
        } catch (e) {
        }
    }

    zs.getDeviceSession = function (object) {
        if (null == object) {
            return;
        }
        let type = object.type;
        if (null == type || '' == type) {
            type = 'ali';
        }
        if (null == deviceSessionResults[type]) {
            deviceSessionResults[type] = [];
        }
        deviceSessionResults[type].push(object);
        try {
            bridge.call("onDeviceSession", type);
        } catch (e) { }
    }

    // 废弃
    zs.onDeviceSession = function (callback) {
        if (!isFunction(callback)) {
            console.error('参数不是一个方法');
            return;
        }
        zs.getDeviceSession({
            type: 'ali', success: (res) => {
                callback({ code: 0, session: res.session });
            }, fail: (res) => {
                callback({ code: res.errCode, session: res.errMsg });
            }
        });
    }

    // 废弃
    zs.offDeviceSession = function (callback) { }

    zs.getSystemInfoSync = function () {
        try {
            let info = bridge.call(`getSystemInfoBase64`);
            return JSON.parse(atob(info));
        } catch (e) {
            return {}
        }
    }

    zs.isInstall = function (name) {
        if (null == name || '' == name) {
            return false;
        }
        try {
            return bridge.call('isInstall', name);
        } catch (e) {
            return false;
        }
    }

    zs.setClipboardData = function (object) {
        if (null == object || null == object.data) {
            object && object.fail && object.fail();
        } else {
            try {
                bridge.call("setClipboardData", object.data);
                object && object.success && object.success();
            } catch (e) {
                object && object.fail && object.fail();
            }
        }
        object && isFunction(object.complete) && object.complete();
    }

    zs.getClipboardData = function (object) {
        if (null == object || !isFunction(object.success)) {
            console.error('回调方法不存在');
            return;
        }
        if (null != object) {
            if (clipboardCallbacks.length > 0) {
                clipboardCallbacks.push(object);
                return;
            }
            try {
                clipboardCallbacks.push(object);
                bridge.call("getClipboardData");
            } catch (e) {
                clipboardCallbacks.splice(clipboardCallbacks.length - 1, 1);
                object && object.fail && object.fail();
                object && object.complete && object.complete();
            }
        }
    }

    zs.vibrateShort = function (object) {
        try {
            bridge.call('vibrate', 15);
            object && isFunction(object.success) && object.success();
        } catch (e) {
            object && isFunction(object.fail) && object.fail();
        }
        object && isFunction(object.complete) && object.complete();
    }

    zs.vibrateLong = function (object) {
        try {
            bridge.call('vibrate', 400);
            object && isFunction(object.success) && object.success();
        } catch (e) {
            object && isFunction(object.fail) && object.fail();
        }
        object && isFunction(object.complete) && object.complete();
    }

    zs.onPolicyResult = function (callback) {
        policyResults.push(callback);
        try {
            bridge.call(`onPolicyResult`);
        } catch (e) { }
    }

    zs.gameLoadResult = function () {
        try {
            bridge.call(`gameLoadResult`);
        } catch (e) { }
    }

    zs.onAppLoadResult = function (callback) {
        if (!isFunction(callback)) {
            console.error('参数不是一个方法');
            return;
        }
        appLoadResults.push(callback);
        try {
            bridge.call(`onAppLoadResult`);
        } catch (e) { }
    }

    zs.offAppLoadResult = function (callback) {
        let idx = appLoadResults.indexOf(callback);
        if (idx === -1) {
            console.warn('没有注册该回调');
        } else {
            appLoadResults.splice(idx, 1);
        }
    }

    zs.onSdkLoginResult = function (callback) {
        if (!isFunction(callback)) {
            console.error('参数不是一个方法');
            return;
        }
        sdkLoginResults.push(callback);
        try {
            bridge.call('onSdkLoginResult');
        } catch (e) { }
    }

    zs.offSdkLoginResult = function (callback) {
        let idx = sdkLoginResults.indexOf(callback);
        if (idx === -1) {
            console.warn('没有注册该回调');
        } else {
            sdkLoginResults.splice(idx, 1);
        }
    }

    zs.reportEvent = function (object) {
        if (null == object) {
            return;
        }
        try {
            let json = JSON.stringify(object);
            bridge.call('reportEvent', json);
        } catch (e) {
        }
    }

    zs.activeEvent = function () {
        try {
            bridge.call(`activeEvent`);
        } catch (e) { }
    }

    zs.registerEvent = function (object) {
        if (null == object) {
            object = {};
        }
        if (null == object.isSuccess) {
            object.isSuccess = true;
        }
        try {
            bridge.call(`registerEvent`, object.pf || "zs", object.userId || '', object.isSuccess);
        } catch (e) { }
    }

    zs.loginEvent = function (object) {
        if (null == object) {
            object = {};
        }
        if (null == object.isSuccess) {
            object.isSuccess = true;
        }
        try {
            bridge.call(`loginEvent`, object.pf || "zs", object.userId || '', object.isSuccess);
        } catch (e) { }
    }

    zs.nextDayStayEvent = function () {
        try {
            bridge.call(`nextDayStayEvent`);
        } catch (e) { }
    }

    zs.weekStayEvent = function () {
        try {
            bridge.call(`weekStayEvent`);
        } catch (e) { }
    }

    zs.payEvent = function (object) {
        if (null == object) {
            object = {};
        }
        if (null == object.isSuccess) {
            object.isSuccess = true;
        }
        if (null == object.virtual) {
            object.virtual = false;
        }
        try {
            bridge.call(`payEvent`, object.userId || '',
                object.contentType || "type",
                object.contentName || "name",
                object.contentId || "id",
                object.contentNumber || 1,
                object.paymentChannel || "zs",
                object.currency || "RMB",
                object.isSuccess || true,
                object.currencyAmount || 1,
                object.virtual);
        } catch (e) { }
    }

    zs.keyEvent = function (object) {
        if (null == object) {
            return;
        }
        if (null == object.eventId) {
            return;
        }
        try {
            bridge.call(`keyEvent`, JSON.stringify(object));
        } catch (e) { }
    }

    // 废弃
    zs.customEvent = zs.keyEvent;

    zs.onEvent = function (eventId, params) {
        try {
            let str = '';
            if (null != params) {
                str = JSON.stringify(params);
            }
            bridge.call(`onEvent`, eventId, str);
        } catch (e) { }
    }

    zs.onCustomEvent = function (object) {
        if (null == object) {
            return;
        }
        try {
            let str = '';
            if (null != object) {
                str = JSON.stringify(object);
            }
            jsb.reflection.callStaticMethod('com/zhise/bridge/ZBSdk', 'onCustomEvent', '(Ljava/lang/String;)V', str);
        } catch (e) { }
    }

    zs.reportError = function (object) {
        if (null == object) {
            return;
        }
        try {
            bridge.call('reportError', object.title, object.info);
        } catch (e) {
        }
    }

    zs.putBuglyUserData = function (key, value) {
        try {
            bridge.call(`putBuglyUserData`, key, value);
        } catch (e) { }
    }

    zs.onEventBegin = function (eventId, params) {
        try {
            let str = '';
            if (null != params) {
                str = JSON.stringify(params);
            }
            bridge.call(`onEventBegin`, eventId, str);
        } catch (e) { }
    }

    zs.onEventEnd = function (eventId, params) {
        try {
            let str = '';
            if (null != params) {
                str = JSON.stringify(params);
            }
            bridge.call(`onEventEnd`, eventId, str);
        } catch (e) { }
    }

    zs.onPageStart = function (pageName) {
        try {
            bridge.call(`onPageStart`, pageName);
        } catch (e) { }
    }

    zs.onPageEnd = function (pageName) {
        try {
            bridge.call(`onPageEnd`, pageName);
        } catch (e) { }
    }

    zs.createBannerAd = function (object) {
        if (null == object) {
            return null;
        }
        if (null == object.style) {
            return null;
        }
        if (null == object.adUnitId) {
            object.adUnitId = '';
        }
        if (null == object.adIntervals) {
            object.adIntervals = 30;
        }
        try {
            let id = bridge.call(`createBannerAd`, object.adUnitId, object.adIntervals, object.style.left, object.style.top, object.style.width, object.style.height);
            bannerAds[id] = new BannerAd(id, object.style);
            return bannerAds[id];
        } catch (e) {
            return null;
        }
    }

    zs.createInterstitialAd = function (object) {
        if (null == object) {
            object = {};
        }
        let adUnitId = object.adUnitId;
        if (null == adUnitId) {
            adUnitId = '';
        }
        if (null != interstitialAds[adUnitId]) {
            return interstitialAds[adUnitId];
        }
        try {
            bridge.call("createInterstitialAd", adUnitId);
            interstitialAds[adUnitId] = new InterstitialAd(adUnitId);
            return interstitialAds[adUnitId];
        } catch (e) {
            return null;
        }
    }

    zs.createRewardedVideoAd = function (object) {
        if (null == object) {
            object = {};
        }
        var adUnitId = object.adUnitId;
        if (null == adUnitId) {
            adUnitId = '';
        }
        let userId;
        if (null == object.userId) {
            userId = '';
        } else {
            userId = object.userId + '';
        }
        if (null != rewardedVideoAds[adUnitId] && userId == rewardedVideoUserIds[adUnitId]) {
            return rewardedVideoAds[adUnitId];
        }
        rewardedVideoUserIds[adUnitId] = userId;
        try {
            bridge.call("createRewardedVideoAd", adUnitId, userId);
            rewardedVideoAds[object.adUnitId] = new RewardedVideoAd(adUnitId);
            return rewardedVideoAds[adUnitId];
        } catch (e) {
            return null;
        }
    }

    zs.createNativeAd = function (object) {
        if (null == object) {
            return null;
        }
        if (null == object.style) {
            return null;
        }
        if (null == object.adUnitId) {
            object.adUnitId = '';
        }
        try {
            let id = bridge.call("createNativeAd", object.adUnitId, object.style.left, object.style.top, object.style.width, object.style.height);
            nativeAds[id] = new NativeAd(id, object.style);
            return nativeAds[id];
        } catch (e) {
            return null;
        }
    }

    zs.login = function (object) {
        if (null == object) {
            return;
        }
        try {
            let did = bridge.call("login");
            object.success && object.success({ code: did });
        } catch (e) {
            object.fail && object.fail();

        }
        object.complete && object.complete();
    }

    zs.logout = function () {
        try {
            bridge.call('logout');
        } catch (e) {
        }
    }

    zs.authorize = function (object) {
        authorizeResult = { success: object.success, fail: object.fail, complete: object.complete };
        try {
            bridge.call("authorize", object.scope || "wx");
        } catch (e) {
            object && object.fail && object.fail();
            object && object.complete && object.complete();
        }
    }

    zs.shareAppMessage = function (object) {
    }

    zs.showNotification = function (object) {
        try {
            bridge.call('showNotification', object.notificationId, object.title || '', object.content || '', object.buttonContent || '', object.jumpActivity || '', object.visibility || 1, object.priority || 2, object.importance || 3);
        } catch (e) {
        }
    }

    window.zsCall = {};

    zsCall.onDeviceSession = function (type, code, msg) {
        if (null == deviceSessionResults[type]) {
            return;
        }
        msg = atob(msg);
        deviceSessionResults[type].forEach((object) => {
            if (0 == code) {
                object.success && object.success({ session: msg });
            } else {
                object.fail && object.fail({ errCode: code, errMsg: msg });
            }
            object.complete && object.complete();
        });
        deviceSessionResults[type].length = 0;
    }

    zsCall.onClipboardData = function (text) {
        clipboardCallbacks.forEach((object) => {
            object && isFunction(object.success) && object.success({ data: text });
        });
        clipboardCallbacks.length = 0;
    }

    zsCall.onPolicyResult = function (bool) {
        setTimeout(() => {
            policyResults.forEach((callback) => {
                callback && callback(bool);
            });
        }, 500);
    }

    zsCall.onAppLoadResult = function () {
        appLoadResults.forEach((object) => {
            object();
        });
    }

    zsCall.onSdkLoginResult = function (uuid, attribute) {
        sdkLoginResults.forEach((object) => {
            object({ uuid: uuid, attribute: attribute });
        });
    }

    zsCall.onAuthor = function (errCode, msg) {
        setTimeout(() => {
            if (0 == errCode) {
                msg = atob(msg);
                authorizeResult && isFunction(authorizeResult.success) && authorizeResult.success({ errCode: 0, code: msg });
            } else {
                authorizeResult && isFunction(authorizeResult.fail) && authorizeResult.fail({ errCode: errCode, errMsg: msg });
            }
            authorizeResult && isFunction(authorizeResult.complete) && authorizeResult.complete();
            authorizeResult = null;
        }, 150);
    }

    function onAdResize(ad, width, height) {
        if (null != ad) {
            ad.resizeCallbacks.forEach((callback) => {
                callback({ width: width, height: height });
            });
        }
    }

    function onAdLoad(ad) {
        if (null != ad) {
            ad.loadCallbacks.forEach((callback) => {
                callback();
            });
        }
    }

    function onAdError(ad, errCode, errMsg) {
        setTimeout(() => {
            if (null != ad) {
                ad.errorCallbacks.forEach((callback) => {
                    callback(errCode, errMsg);
                });
            }
        }, 150);
    }

    function adLoadResolve(ad) {
        if (null != ad) {
            let loadResolves = ad.loadResolves;
            ad.loadResolves = [];
            ad.loadRejects.length = 0;
            loadResolves.forEach((resolve) => {
                resolve();
            });
            loadResolves = null;
        }
    }

    function adLoadReject(ad) {
        setTimeout(() => {
            if (null != ad) {
                let loadRejects = ad.loadRejects;
                ad.loadRejects = [];
                ad.loadResolves.length = 0;
                loadRejects.forEach((reject) => {
                    reject();
                });
            }
        });
    }

    function adShowResolve(ad) {
        setTimeout(() => {
            if (null != ad) {
                let showResolves = ad.showResolves;
                ad.showResolves = [];
                ad.showRejects.length = 0;
                showResolves.forEach((resolve) => {
                    resolve();
                });
            }
        }, 150);
    }

    function adShowReject(ad) {
        setTimeout(() => {
            if (null != ad) {
                ad.showRejects.forEach((reject) => {
                    reject();
                });
                ad.showResolves.length = 0;
                ad.showRejects.length = 0;
            }
        }, 150);
    }

    function onAdClick(ad) {
        setTimeout(() => {
            if (null != ad) {
                let showResolves = ad.showResolves;
                ad.showResolves = [];
                ad.showRejects.length = 0;
                showResolves.forEach((resolve) => {
                    resolve();
                });
            }
        }, 150);
    }

    zsCall.onBannerAdResize = function (id, width, height) {
        onAdResize(bannerAds[id], width, height);
    }

    zsCall.onBannerAdLoad = function (id) {
        onAdLoad(bannerAds[id]);
    }

    zsCall.onBannerAdLoadError = function (id, errCode, errMsg) {
        onAdError(bannerAds[id], errCode, errMsg);
    }

    zsCall.onBannerAdShow = function (id) {
    }

    zsCall.onBannerAdShowError = function (id, errCode, errMsg) {
        onAdError(bannerAds[id], errCode, errMsg);
    }

    zsCall.onInterstitialAdClose = function (adUnitId) {
        setTimeout(() => {
            let ad = interstitialAds[adUnitId];
            if (null != ad) {
                ad.closeCallbacks.forEach((callback) => {
                    callback({});
                });
            }
        }, 150);
    }

    zsCall.onInterstitialAdLoad = function (adUnitId) {
        onAdLoad(interstitialAds[adUnitId]);
        adLoadResolve(interstitialAds[adUnitId]);
    }

    zsCall.onInterstitialAdLoadError = function (adUnitId, errCode, errMsg) {
        onAdError(interstitialAds[adUnitId], errCode, errMsg);
        adLoadReject(interstitialAds[adUnitId]);
    }

    zsCall.onInterstitialAdShow = function (adUnitId) {
        adShowResolve(interstitialAds[adUnitId]);
    }

    zsCall.onInterstitialAdShowError = function (adUnitId, errCode, errMsg) {
        onAdError(interstitialAds[adUnitId], errCode, errMsg);
        adShowReject(interstitialAds[adUnitId]);
    }

    zsCall.onInterstitialAdClick = function (adUnitId) {
        onAdClick(interstitialAds[adUnitId]);
    }

    zsCall.onRewardedVideoAdClose = function (adUnitId, verify, ecpm) {
        setTimeout(() => {
            let ad = rewardedVideoAds[adUnitId];
            if (null != ad) {
                ad.closeCallbacks.forEach((callback) => {
                    callback({ isEnded: verify, ecpm: ecpm });
                });
            }
        }, 150);
    }

    zsCall.onRewardedVideoAdLoad = function (adUnitId) {
        onAdLoad(rewardedVideoAds[adUnitId]);
        adLoadResolve(rewardedVideoAds[adUnitId]);
    }

    zsCall.onRewardedVideoAdLoadError = function (adUnitId, errCode, errMsg) {
        onAdError(rewardedVideoAds[adUnitId], errCode, errMsg);
        adLoadReject(rewardedVideoAds[adUnitId]);
    }

    zsCall.onRewardedVideoAdShow = function (adUnitId) {
        adShowResolve(rewardedVideoAds[adUnitId]);
    }

    zsCall.onRewardedVideoAdShowError = function (adUnitId, errCode, errMsg) {
        onAdError(rewardedVideoAds[adUnitId], errCode, errMsg);
        adShowReject(rewardedVideoAds[adUnitId]);
    }

    zsCall.onRewardedVideoAdClick = function (adUnitId) {
        onAdClick(rewardedVideoAds[adUnitId]);
    }

    zsCall.onNativeAdResize = function (id, width, height) {
        onAdResize(nativeAds[id], width, height);
    }

    zsCall.onNativeAdLoad = function (id) {
        onAdLoad(nativeAds[id]);
    }

    zsCall.onNativeAdLoadError = function (id, errCode, errMsg) {
        onAdError(nativeAds[id], errCode, errMsg);
    }

    zsCall.onNativeAdShow = function (id) {
    }

    zsCall.onNativeAdShowError = function (id, errCode, errMsg) {
        onAdError(nativeAds[id], errCode, errMsg);
    }

    let _keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";

    // private method for UTF-8 encoding
    function _utf8_encode(string) {
        string = string.replace(/\r\n/g, "\n");
        let utftext = '';
        for (let n = 0; n < string.length; n++) {
            let c = string.charCodeAt(n);
            if (c < 128) {
                utftext += String.fromCharCode(c);
            } else if ((c > 127) && (c < 2048)) {
                utftext += String.fromCharCode((c >> 6) | 192);
                utftext += String.fromCharCode((c & 63) | 128);
            } else {
                utftext += String.fromCharCode((c >> 12) | 224);
                utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                utftext += String.fromCharCode((c & 63) | 128);
            }

        }
        return utftext;
    }

    // private method for UTF-8 decoding
    function _utf8_decode(utftext) {
        let string = '';
        let i = 0;
        let c = 0;
        let c1 = 0;
        let c2 = 0;
        let c3 = 0;
        while (i < utftext.length) {
            c = utftext.charCodeAt(i);
            if (c < 128) {
                string += String.fromCharCode(c);
                i++;
            } else if ((c > 191) && (c < 224)) {
                c2 = utftext.charCodeAt(i + 1);
                string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
                i += 2;
            } else {
                c2 = utftext.charCodeAt(i + 1);
                c3 = utftext.charCodeAt(i + 2);
                string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
                i += 3;
            }
        }
        return string;
    }

    function btoa(input) {
        let output = '';
        let chr1, chr2, chr3, enc1, enc2, enc3, enc4;
        let i = 0;
        input = _utf8_encode(input);
        while (i < input.length) {
            chr1 = input.charCodeAt(i++);
            chr2 = input.charCodeAt(i++);
            chr3 = input.charCodeAt(i++);
            enc1 = chr1 >> 2;
            enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
            enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
            enc4 = chr3 & 63;
            if (isNaN(chr2)) {
                enc3 = enc4 = 64;
            } else if (isNaN(chr3)) {
                enc4 = 64;
            }
            output = output +
                _keyStr.charAt(enc1) + _keyStr.charAt(enc2) +
                _keyStr.charAt(enc3) + _keyStr.charAt(enc4);
        }
        return output;
    }

    function atob(input) {
        let output = '';
        let chr1, chr2, chr3;
        let enc1, enc2, enc3, enc4;
        let i = 0;
        input = input.replace(/[^A-Za-z0-9\+\/\=]/g, '');
        while (i < input.length) {
            enc1 = _keyStr.indexOf(input.charAt(i++));
            enc2 = _keyStr.indexOf(input.charAt(i++));
            enc3 = _keyStr.indexOf(input.charAt(i++));
            enc4 = _keyStr.indexOf(input.charAt(i++));
            chr1 = (enc1 << 2) | (enc2 >> 4);
            chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
            chr3 = ((enc3 & 3) << 6) | enc4;
            output = output + String.fromCharCode(chr1);
            if (enc3 != 64) {
                output = output + String.fromCharCode(chr2);
            }
            if (enc4 != 64) {
                output = output + String.fromCharCode(chr3);
            }
        }
        output = _utf8_decode(output);
        return output;
    }

})();